@@ -19,11 +19,15 @@
 """Provides classes for interacting with magnetic equilibrium data in a variety of formats.
 """
 
+__version__ = '1.4.0'
+
 from .core import *
-from .EFIT import *
-from .CModEFIT import *
-from .NSTXEFIT import *
-from .D3DEFIT import *
-from .eqdskreader import *
-from .pfilereader import *
-from .FromArrays import *
+# from .EFIT import *
+# from .CModEFIT import *
+# from .AUGData import *
+# from .NSTXEFIT import *
+# from .D3DEFIT import *
+# from .eqdskreader import *
+# from .pfilereader import *
+# from .FromArrays import *
+# from .TCVLIUQE import *
